' AT45 controller for DDS AD9850 on china break-out board
'
' with RC5 remote control for DDS module frequency 
' incl. frequency scan or sweep 100 steps up
'
' DDS on Portb 0 1 2
' Portb.2 = WCLK
' Portb.1 = FQUD
' Portb.0 = DATA
' Portb.4 = serial out
' Portb.3 = IR RC5 input
' GS 2015/2
'
'***************************************************************************
'  This program is free software; you can redistribute it and/or
'  modify it under the terms of the GNU General Public License.
'  This program is distributed in the hope that it will be useful,
'  but WITHOUT ANY WARRANTY;
'***************************************************************************
'
'                            ATtiny45 dip
' (PCINT5/RESET/ADC0/dW)         PB5  VCC
' (PCINT3/XTAL1/CLKI/OC1B/ADC3)  PB3  PB2 (SCK/USCK/SCL/ADC1/T0/INT0/PCINT2)
' (PCINT4/XTAL2/CLKO/OC1B/ADC2)  PB4  PB1 (MISO/DO/AIN1/OC0B/OC1A/PCINT1)
'                                GND  PB0 (MOSI/DI/SDA/AIN0/OC0A/OC1A/AREF/PCINT0)
'
';***************************************************************************
' PB0 output  DATA
' PB1 output  FQUD
' PB2 output  WCLK
' PB4 output  Com1
' PB3 input   IR RC5
'***************************************************************************

$regfile = "attiny45.dat"
$crystal = 1000000
'$crystal = 8000000
$hwstack = 8
$swstack = 8
$framesize = 4


Wdat Alias Portb.0                                          'AD9850 Wdat     output
Fqud Alias Portb.1                                          'AD9850 FQUD     output
Wclk Alias Portb.2                                          'AD9850 SCLK      output

Const Clk_in = 125000000
Const Multi = &H100000000
Const F_max = 40000000                                      ' 40 MHz

Dim Address As Byte , Command As Byte , Tog As Byte
Dim T0 As Byte , T1 As Byte , N As Byte , Dds_cmd As Byte
Dim Flagscan As Byte , FlagRS232 As Byte, I As Byte
Dim Instring As String * 12
Dim Freq_data As Long , Freq_in As Long
Dim F_in1 As Single , F_in2 As Single
Dim F_inold As Single , F_step As Single
Dim Freq_accu As Single

Portb = &B00001000
Ddrb = &B00010111

'CLKPR = &B10000000 'Clock Prescaler Change Enable
'CLKPR = &B00000000 'Clock Division Factor=0, 8 MHz

Config Rc5 = Pinb.3

Acsr.acd = 0                                                ' switch off analog comparator

Open "comb.4:9600,8,n,1,INVERTED" For Output As #1

Instring = ""                                               'blank
F_in1 = 100000                                              '100 KHz
F_in2 = 120000                                              '120 KHz
F_inold = 100000                                            '100 KHz
F_step = 1000
Freq_in = 100000
Flagscan = 0
FlagRS232 = 0

Dds_cmd = &B00000000

Gosub Init_dds
Waitms 100
Gosub Dds_output

Print #1 , "Wait for RC5"
Enable Interrupts

Do
  Getrc5(address , Command)
  If Address < 255 Then
     'the toggle bit toggles on each new received command
     'toggle bit is bit 7. Extended RC5 bit is in bit 6
     Tog = Command And &B10000000
     Command = Command And &B01111111                       'clear the toggle bit

     'Disable Interrupts
     'Print #1 , " A:" ; Address ; " C:" ; Command ; " T:" ; Tog
     'Enable Interrupts
     T0 = Tog
     If Command <= 9 And T0 <> T1 Then
        N = Command + 48                                    'make ascii number
        Instring = Instring + Chr(n)
        T1 = T0                                             'store numbers
     Else
        T1 = T0
     End If

    Select Case Command
     Case 38 :                                              'set F1
                Freq_in = F_in1
                Gosub Chkvalid
                F_in1 = Freq_in
     Case 30 :                                              'set F2
                Freq_in = F_in2
                Gosub Chkvalid
                F_in2 = Freq_in
     Case 34 :
                Freq_in = F_step                            'set F_step
                Gosub Chkvalid
                F_step = Freq_in
     Case 16 :
                Flagscan = 1                                   'start scan

     Case 17 :
                Flagscan = 0                                   'stop scan

     Case 15 :
                FlagRS232 = 1                                   'start RS232 output

     Case 12 :
                FlagRS232 = 0                                   'stop RS232

     Case 32 :                                              ' + Command
                F_in1 = F_in1 + F_step
                If F_in1 > F_max Then
                   F_in1 = F_max
                End If
                Gosub Chkvalid
     Case 33 :                                              ' - Command
                F_in1 = F_in1 - F_step
                If F_in1 < 0 Then
                   F_in1 = 0
                End If
                Gosub Chkvalid
    End Select

   Disable Interrupts                                       
   Enable Interrupts

  End If                                                    'If Address < 255

  If Flagscan = 1 Then                                         'scan
     F_inold = F_in1
     For I = 1 To 100
       F_in2 = I * F_step
       F_in1 = F_inold + F_in2
       Gosub Dds_output                                     'and send to DDS
       Waitms 1
     Next
     Flagscan = 0
     F_in1 = F_inold
  Else                                                      'no scan
  If F_in1 <> F_inold Then                                  'only send changes
     Gosub Dds_output                                       'and send to DDS
     F_inold = F_in1
   End If                                                   'If F_in1 <> F_in1old
  End If                                                    'If Flagscan = 1


Loop

'***************************************************************************
' check length of string
'***************************************************************************
Chkvalid:

   If Len(instring) >= 1 And Len(instring) <= 8 Then
     Freq_in = Val(instring)
   End If

   Instring = ""                                            ''                                         '';
   T1 = T0
Return

'***************************************************************************
' Init_dds
'***************************************************************************
Init_dds:

   Reset Wdat                                               'DATA                                '
   Reset Wclk                                               'WCLK
   Waitus 10
   Set Wclk                                                 'WCLK
   Waitus 10
   Reset Wclk                                               'WCLK
   Waitus 10
   Set Fqud                                                 'FQUD
   Waitus 10
   Reset Fqud                                               'FQUD
   Waitus 10
Return

'***************************************************************************
' Dds_output
'***************************************************************************
Dds_output:

   Disable Interrupts
   
   Freq_accu = F_in1 * Multi                                'calculate
   Freq_accu = Freq_accu / Clk_in
   Freq_data = Freq_accu

   Shiftout Wdat , Wclk , Freq_data , 3 , 32 , 0            'DATA,WCLK
   Shiftout Wdat , Wclk , Dds_cmd , 3 , 8 , 0               'DATA,WCLK
   Waitus 10
   Set Fqud                                                 'FQUD
   Waitus 10
   Reset Fqud                                               'FQUD
  
   If FlagRS232 = 1 Then	'debug print
     Print #1 , " Cmd:" ; Command ; " In:" ; Instring
     Print #1 , " F_in1:" ; F_in1 ; " F_in2:" ; F_in2 ; " F_st:" ; F_step
     Print #1 , " F_old:" ; F_inold ; " F_in:" ; Freq_in
   End If	
   
   Enable Interrupts
Return

End